#!/usr/bin/env python

from os.path import dirname, join, realpath
from setuptools import setup, find_packages, Extension

from Cython.Build import cythonize
from karabo.packaging.versioning import device_scm_version


ROOT_FOLDER = dirname(realpath(__file__))
scm_version = device_scm_version(
    ROOT_FOLDER, join(ROOT_FOLDER, "src", "calng", "_version.py")
)


setup(
    name="calng",
    use_scm_version=scm_version,
    author="CAL team",
    author_email="da-support@xfel.eu",
    description="",
    long_description="",
    url="",
    package_dir={"": "src"},
    packages=find_packages("src"),
    entry_points={
        "karabo.bound_device": [
            "AgipdCorrection = calng.corrections.AgipdCorrection:AgipdCorrection",
            "DsscCorrection = calng.corrections.DsscCorrection:DsscCorrection",
            "Epix100Correction = calng.corrections.Epix100Correction:Epix100Correction",  # noqa
            "Gotthard2Correction = calng.corrections.Gotthard2Correction:Gotthard2Correction",  # noqa
            "JungfrauCorrection = calng.corrections.JungfrauCorrection:JungfrauCorrection",  # noqa
            "LpdCorrection = calng.corrections.LpdCorrection:LpdCorrection",
            "LpdminiCorrection = calng.corrections.LpdminiCorrection:LpdminiCorrection",
            "PnccdCorrection = calng.corrections.PnccdCorrection:PnccdCorrection",
            "ShmemToZMQ = calng.ShmemToZMQ:ShmemToZMQ",
            "ShmemTrainMatcher = calng.ShmemTrainMatcher:ShmemTrainMatcher",
            "FrameSelectionArbiter = calng.FrameSelectionArbiter:FrameSelectionArbiter",  # noqa
            "DetectorAssembler = calng.DetectorAssembler:DetectorAssembler",
            "Gotthard2Assembler = calng.Gotthard2Assembler:Gotthard2Assembler",
            "LpdminiSplitter = calng.LpdminiSplitter:LpdminiSplitter",
        ],
        "karabo.middlelayer_device": [
            "CalibrationManager = calng.CalibrationManager:CalibrationManager",
            "AgipdCondition = calng.conditions:AgipdCondition.AgipdCondition",
            "JungfrauCondition = calng.conditions.JungfrauCondition:JungfrauCondition",  # noqa
            "LpdCondition = calng.conditions.LpdCondition:LpdCondition",
            "RoiTool = calng.RoiTool:RoiTool",
        ],
        "calng.correction_addon": [
            "IntegratedIntensity = calng.correction_addons.integrated_intensity:IntegratedIntensity",  # noqa
            "LitPixelCounter = calng.correction_addons.litpixel_counter:LitPixelCounter [agipd]",  # noqa
            "Peakfinder9 = calng.correction_addons.peakfinder9:Peakfinder9",  # noqa
            "RandomFrames = calng.correction_addons.random_frames:RandomFrames",  # noqa
        ],
        "calng.arbiter_kernel": [
            "BooleanCombination = calng.arbiter_kernels.boolean_ops:BooleanCombination",  # noqa
            "Ppu = calng.arbiter_kernels.ppu_arbiter:PpuKernel",
            "RandomFrames = calng.arbiter_kernels.random_frames:RandomSampler",  # noqa
            "ReduceAndThreshold = calng.arbiter_kernels.reduce_threshold:ReduceAndThreshold",  # noqa
        ],
    },
    package_data={"": ["kernels/*"]},
    requires=[],
    ext_modules=cythonize(
        [
            Extension(
                "calng.kernels.agipd_cython",
                ["src/calng/kernels/agipd_cpu.pyx"],
                extra_compile_args=["-O3", "-march=native", "-fopenmp"],
                extra_link_args=["-fopenmp"],
            ),
            Extension(
                "calng.kernels.common_cython",
                ["src/calng/kernels/common_cpu.pyx"],
                extra_compile_args=["-O3", "-march=native", "-fopenmp"],
                extra_link_args=["-fopenmp"],
                language="c++",
            ),
            Extension(
                "calng.kernels.dssc_cython",
                ["src/calng/kernels/dssc_cpu.pyx"],
                extra_compile_args=["-O3", "-march=native", "-fopenmp"],
                extra_link_args=["-fopenmp"],
            ),
            Extension(
                "calng.kernels.gotthard2_cython",
                ["src/calng/kernels/gotthard2_cpu.pyx"],
                extra_compile_args=["-O3", "-march=native"],
            ),
            Extension(
                "calng.kernels.jungfrau_cython",
                ["src/calng/kernels/jungfrau_cpu.pyx"],
                extra_compile_args=["-O3", "-march=native", "-fopenmp"],
                extra_link_args=["-fopenmp"],
            ),
            Extension(
                "calng.kernels.lpd_cython",
                ["src/calng/kernels/lpd_cpu.pyx"],
                extra_compile_args=["-O3", "-march=native", "-fopenmp"],
                extra_link_args=["-fopenmp"],
            ),
        ],
        compiler_directives={"language_level": 3},
    ),
)
