from karabo.middlelayer import AccessMode, Assignment, String
from .. import base_condition
from ..corrections.AgipdCorrection import GainModes


class AgipdCondition(base_condition.ConditionBase):
    controlDeviceId = String(
        displayedName="Control device ID",
        assignment=Assignment.MANDATORY,
        accessMode=AccessMode.INITONLY,
    )

    voltageSourceDevice = String()
    voltageSourceKey = String()

    @property
    def keys_to_get(self):
        return {
            self.controlDeviceId.value: [
                # rep rate: 1.1, 2.2, 4.5
                ("bunchStructure.repetitionRate", "acquisitionRate", None),
                # TODO: check if appropriate (agipdlib looks at image.cellId)
                ("bunchStructure.nPulses", "memoryCells", None),
                ("gain", "gainSetting", None),
                (
                    "gainModeIndex",
                    "gainMode",
                    lambda i: GainModes(i).name,
                ),
                ("integrationTime", "integrationTime", None),
            ],
            # observed voltages in calcat:
            # 59.0
            # 64.0
            # 74.0
            # 100.0
            # 199.0
            # 200.0
            # 300.0
            # 500.0
            self.voltageSourceDevice.value: [
                (
                    self.voltageSourceKey.value,
                    "biasVoltage",
                    lambda n: round(n, ndigits=0),
                )
            ],
        }
