# cython: boundscheck=False
# cython: cdivision=True
# cython: wrapararound=False

from cython.parallel import prange
import numpy as np


cdef extern from "<algorithm>" namespace "std" nogil:
    void nth_element[Iter](Iter first, Iter nth, Iter last) except +


def cm_fs(
    float[:, :] data,
    unsigned char[:, :] mask,
    float sigma,
    float ratio,
):
    # aux will essentially contain the unmasked data
    # used in median finding which reorders input, hence copying is needed anyway
    cdef float[:, :] aux = np.empty_like(data, order="C")
    cdef int num_dark, min_dark, i, j, k
    cdef float median
    min_dark = <int>(<float>(data.shape[1]) * ratio)
    for i in prange(data.shape[0], nogil=True):
        num_dark = 0
        for j in range(data.shape[1]):
            if mask[i, j] == 0:
                aux[i, num_dark] = data[i, j]
                num_dark = num_dark + 1
        if num_dark < min_dark:
            continue
        k = num_dark // 2
        nth_element(&aux[i, 0], (&aux[i, 0]) + k, (&aux[i, 0]) + num_dark)
        median = aux[i, k]
        if num_dark % 2 == 0:
            nth_element(&aux[i, 0], (&aux[i, 0]) + k - 1, (&aux[i, 0]) + num_dark)
            median = median / 2 + aux[i, k - 1] / 2

        for j in range(data.shape[1]):
            data[i, j] = data[i, j] - median


def cm_ss(
    float[:, :] data,
    unsigned char[:, :] mask,
    float sigma,
    float ratio,
):
    # transposing to give std::nth_element contiguous input
    cdef float[:, ::1] aux = np.empty_like(np.transpose(data), order="C")
    cdef int num_dark, min_dark, i, j, k
    cdef float median
    min_dark = <int>(<float>data.shape[1] * ratio)
    for j in prange(data.shape[1], nogil=True):
        num_dark = 0
        for i in range(data.shape[0]):
            if mask[i, j] == 0:
                aux[j, num_dark] = data[i, j]
                num_dark = num_dark + 1
        if num_dark < min_dark:
            continue
        k = num_dark // 2
        nth_element(&aux[j, 0], (&aux[j, 0]) + k, (&aux[j, 0]) + <int>num_dark)
        median = aux[j, k]
        if num_dark % 2 == 0:
            nth_element(&aux[j, 0], (&aux[j, 0]) + k - 1, (&aux[j, 0]) + <int>num_dark)
            median = median / 2 + aux[j, k - 1] / 2
        for i in range(data.shape[0]):
            data[i, j] = data[i, j] - median
